@@ -7,7 +7,7 @@ from ._solvers import _lp_solver
 
 class MADAnalyzer(_RiskAnalyzer):
     """
-    MAD risk measure based portfolio optimization.
+    m-level MAD based optimal portfolio strategies.
 
     Methods:
         * getWeights
@@ -19,55 +19,53 @@ class MADAnalyzer(_RiskAnalyzer):
         * set_rtype
         * set_random_seed
     """
-    def __init__(self, coef=[1.],
-                 mktdata=None, colname='adjusted', freq='Q',
-                 hlength=3.25, calendar=None,
-                 rtype='Sharpe', method='ecos'):
+    def __init__(self, coef=[1.], mktdata=None, colname='adjusted', freq='Q',
+                 hlength=3.25, calendar=None, rtype='Sharpe', method='ecos'):
         """
         Constructor
 
         Parameters
         ----------
-        coef : list, optional
-            List of coefficients (the list size defines the MAD
-            order).The default is [1.].
-        mktdata : pandas.DataFrame, optional
+        `coef` : list, optional
+            Positive non-increasing list of mixture coefficients. 
+            The default is `[1.]`.
+        `mktdata` : `pandas.DataFrame`, optional
             Historic daily market data for portfolio components in the format
-            returned by azapy.mktData function. The default is None.
-        colname : string, optional
+            returned by `azapy.mktData` function. The default is `None`.
+        `colname` : str, optional
             Name of the price column from mktdata used in the weights
-            calibration. The default is 'adjusted'.
-        freq : string, optional
-            Rate of returns horizon in number of business day. it could be
-            'Q' for quarter or 'M' for month. The default is 'Q'.
-        hlength : float, optional
+            calibration. The default is `'adjusted'`.
+        `freq` : str, optional
+            Rate of return horizon in number of business day. it could be
+            'Q' for quarter or 'M' for month. The default is `'Q'`.
+        `hlength` : float, optional
             History length in number of years used for calibration. A
             fractional number will be rounded to an integer number of months.
-            The default is 3.25 years.
-        calendar : np.busdaycalendar, optional
-            Business days calendar. If is it None then the calendar will be set
-            to NYSE business calendar.
-            The default is None.
-        rtype : string, optional
+            The default is `3.25` years.
+        `calendar` : `numpy.busdaycalendar`, optional
+            Business days calendar. If is it `None` then the calendar will
+            be set to NYSE business calendar.
+            The default is `None`.
+        `rtype` : str, optional
             Optimization type. Possible values \n
-                "Risk" : minimization of dispersion (risk) measure for a fixed 
-                vale of expected rate of return. \n
-                "Sharpe" : maximization of generalized Sharpe ratio.\n
-                "Sharpe2" : minimization of the inverse generalized Sharpe 
+                'Risk' : minimization of dispersion (risk) measure for 
+                targeted rate of return. \n
+                'Sharpe' : maximization of generalized Sharpe ratio.\n
+                'Sharpe2' : minimization of the inverse generalized Sharpe 
                 ratio.\n
-                "MinRisk" : optimal portfolio with minimum dispersion (risk) 
-                value.\n
-                "InvNRisk" : optimal portfolio with the same dispersion (risk)
-                value as equal weighted portfolio. 
-                "RiskAverse" : optimal portfolio for a fixed value of risk 
-                aversion coefficient.
-            The default is "Sharpe".
-        method : string, optional
+                'MinRisk' : minimum dispersion (risk) portfolio.\n
+                'InvNrisk' : optimal portfolio with the same dispersion (risk)
+                value as a benchmark portfolio 
+                (e.g. equal weighted portfolio).\n
+                'RiskAverse' : optimal portfolio for a fixed value of 
+                risk-aversion factor.
+            The default is `'Sharpe'`.
+        `method` : str, optional
             method : string, optional
             Linear programming numerical method.
-            Could be one of 'ecos', 'highs-ds', 'highs-ipm', 'highs',
+            Could be:'ecos', 'highs-ds', 'highs-ipm', 'highs',
             'interior-point', 'glpk' and 'cvxopt'.
-            The defualt is 'ecos'.
+            The defualt is `'ecos'`.
 
         Returns
         -------
@@ -76,21 +74,25 @@ class MADAnalyzer(_RiskAnalyzer):
         """
         super().__init__(mktdata, colname, freq, hlength, calendar, rtype)
 
-        lp_methods = ['ecos', 'highs-ds', 'highs-ipm', 'highs',
-                       'interior-point', 'glpk', 'cvxopt']
-        if not method in lp_methods:
-            raise ValueError(f"method must be one of {lp_methods}")
-        self.method = method
+        self._set_method(method)
 
         self.coef = np.array(coef)
         if any(self.coef <= 0.):
             raise ValueError("All coef must be positive")
+        if any(i < j for i, j in zip(self.coef, self.coef[1:])):
+            raise ValueError("coef list should not be increasing")
+            
         self.ll = self.coef.size
+        
         if self.ll <= 0:
             raise ValueError("coef must contain at least one element")
+            
         self.coef = self.coef / np.sum(self.coef)
+        
+        
+    def _set_method(self, method):
+        self._set_lp_method(method)
 
-        self.alpha = np.full(self.ll, self.ll)
 
     def getRisk(self, ww, rrate=None):
         """
@@ -98,17 +100,17 @@ class MADAnalyzer(_RiskAnalyzer):
 
         Parameters
         ----------
-        ww : list (numpy.array or pandas.Series)
+        `ww` : list (`numpy.array` or `pandas.Series`)
             Portfolio weights.
-        rrate : pandas.series, optional
+        `rrate` : `pandas.series`, optional
             The portfolio components historical rates of returns.
-            If it is not None, it will overwrite the rrate computed in the
-            constructor from mktdata. The default is None.
+            If it is not `None`, it will overwrite the rrate computed in the
+            constructor from mktdata. The default is `None`.
 
         Returns
         -------
-        float
-        The value of MAD
+        float :
+        The value of mMAD
         """
         if rrate is not None:
             self.set_rrate(rrate)
@@ -126,11 +128,12 @@ class MADAnalyzer(_RiskAnalyzer):
 
         return self.risk
 
+
     def _risk_calc_(self, prate):
         # mu = np.mean(prate)
         # prate = prate - mu
         # nn = len(prate)
-
+ 
         delta = []
         mu = 0.
         for _ in range(self.ll):
@@ -141,13 +144,13 @@ class MADAnalyzer(_RiskAnalyzer):
             mu -= dd
 
         self.primary_risk_comp = np.array(delta)
-        self.secondary_risk_comp = np.cumsum(self.primary_risk_comp) \
-            -self.primary_risk_comp[0]
+        self.secondary_risk_comp = \
+            np.cumsum(np.insert(self.primary_risk_comp, 0, 0))[:-1]
         self.risk = np.dot(self.primary_risk_comp, self.coef)
 
         return self.risk
 
-
+    
     def _risk_min(self, d=1):
         # Order of variables:
         # w <- [0:mm]
@@ -171,14 +174,15 @@ class MADAnalyzer(_RiskAnalyzer):
         for l in range(ll):
             G_icol += [mm + k * (nn + 1) for k in range(l)] * nn \
                   + list(range(mm + l * (nn + 1) + 1, mm + (l + 1) * (nn + 1)))
-            #irow += list(range(l * nn, (l + 1) * nn)) * (l + 1)
             G_irow += [k for k in range(l * nn, (l + 1) * nn) \
                        for _ in range(l)]\
                     + list(range(l * nn, (l + 1) * nn))
             G_data += [-1.] * ((l + 1) * nn)
+            
         G_icol += list(range(mm))
         G_irow += [ll * nn] * mm
         G_data += list(-self.muk * d)
+        
         G_icol += list(range(mm + ll * (nn + 1)))
         G_irow += list(range(ll * nn + 1, ll * nn + 1 + mm + ll * (nn + 1)))
         G_data += [-1.] * (mm + ll * (nn + 1))
@@ -191,39 +195,36 @@ class MADAnalyzer(_RiskAnalyzer):
                + [0.] * (mm + ll * (nn + 1))
 
         # build A
-        A_icol = []
-        A_irow = []
-        A_data = []
+        A_icol = list(range(mm))
+        A_irow = [0] * mm
+        A_data = [1.] * mm
         for l in range(ll):
             A_icol += list(range(mm + l * (nn + 1), mm + (l + 1) * (nn + 1)))
-            A_irow += [l] * (nn + 1)
-            A_data += [-1.] + [1. / nn] * nn
-        A_icol += list(range(mm))
-        A_irow += [ll] * mm
-        A_data += [1.] * mm
+            A_irow += [l + 1] * (nn + 1)
+            A_data += [-1.] + [1./nn] * nn
 
         A_shape = (ll + 1, mm + (nn + 1) * ll)
         A = sps.coo_matrix((A_data, (A_irow, A_icol)), A_shape)
 
         # build b
-        b_data = [0.] * ll + [1.]
+        b_data = [1.] + [0.] * ll
 
         # calc
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
 
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"warning {res['status']}: {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']}")
             return np.array([np.nan] * mm)
 
         # mMAD
         self.risk = res['pcost']
-        # MAD
+        # delta-risk values
         self.primary_risk_comp = np.array([res['x'][mm + l * (nn + 1)] \
                                            for l in range(ll)])
-        # tMAD
-        self.secondary_risk_comp = np.cumsum(self.primary_risk_comp) \
-            - self.primary_risk_comp[0]
+        # delta-risk strikes
+        self.secondary_risk_comp = \
+            np.cumsum(np.insert(self.primary_risk_comp, 0, 0))[:-1]
         # optimal weights
         self.ww = np.array(res['x'][:mm])
         self.ww.shape = mm
@@ -232,6 +233,7 @@ class MADAnalyzer(_RiskAnalyzer):
 
         return self.ww
 
+
     def _sharpe_max(self):
         # Order of variables:
         # w <- [0:mm]
@@ -255,9 +257,8 @@ class MADAnalyzer(_RiskAnalyzer):
             G_icol += [mm + k * (nn + 1) for k in range(l)] * nn \
                   + list(range(mm + l * (nn + 1) + 1, mm + (l + 1) * (nn + 1)))
             G_irow += list(range(l * nn, (l + 1) * nn)) * (l + 1)
-            # irow += [k for k in range(l * nn, (l + 1) * nn) for _ in range(l)]\
-            #     + list(range(l * nn, (l + 1) * nn))
             G_data += [-1.] * ((l + 1) * nn)
+            
         G_icol += list(range(mm + ll * (nn + 1) + 1))
         G_irow += list(range(ll * nn, ll * nn + mm + ll * (nn + 1) + 1))
         G_data += [-1.] * (mm + ll * (nn + 1) + 1)
@@ -269,16 +270,14 @@ class MADAnalyzer(_RiskAnalyzer):
         h_data = [0.] * (nn * ll + mm + ll * (nn + 1) + 1)
 
         # build A
-        A_icol = []
-        A_irow = []
-        A_data = []
+        A_icol = [mm + l * (nn + 1) for l in range(ll)]
+        A_irow = [0] * ll
+        A_data = list(self.coef)
         for l in range(ll):
             A_icol += list(range(mm + l * (nn + 1), mm + (l + 1) * (nn + 1)))
-            A_irow += [l] * (nn + 1)
+            A_irow += [l + 1] * (nn + 1)
             A_data += [-1.] + [1. / nn] * nn
-        A_icol += [mm + l * (nn + 1) for l in range(ll)]
-        A_irow += [ll] * ll
-        A_data += list(self.coef)
+        
         A_icol += list(range(mm)) + [mm + ll * (nn + 1)]
         A_irow += [ll + 1] * (mm + 1)
         A_data += [1.] * mm + [-1.]
@@ -287,27 +286,27 @@ class MADAnalyzer(_RiskAnalyzer):
         A = sps.coo_matrix((A_data, (A_irow, A_icol)), A_shape)
 
         # build b
-        b_data = [0.] * ll + [1., 0.]
+        b_data = [1.] + [0.] * (ll + 1)
 
         # calc
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
 
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"warning {res['status']}: {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']}")
             return np.array([np.nan] * mm)
 
         t = res['x'][-1]
-        # sharpe
+        # mMAD-Sharpe
         self.sharpe = -res['pcost']
         # mMAD
         self.risk = 1 / t
-        # MAD
+        # delta-risk values
         self.primary_risk_comp = np.array([res['x'][mm + l * (nn + 1)] / t \
                                            for l in range(ll)])
-        # tMAD
-        self.secondary_risk_comp = np.cumsum(self.primary_risk_comp) \
-            - self.primary_risk_comp[0]
+        # delta-risk strikes
+        self.secondary_risk_comp = \
+            np.cumsum(np.insert(self.primary_risk_comp, 0, 0))[:-1]
         # optimal weights
         self.ww = np.array(res['x'][:mm] / t)
         self.ww.shape = mm
@@ -316,6 +315,7 @@ class MADAnalyzer(_RiskAnalyzer):
 
         return self.ww
 
+
     def _sharpe_inv_min(self):
         # Order of variables:
         # w <- [0:mm]
@@ -341,10 +341,11 @@ class MADAnalyzer(_RiskAnalyzer):
         for l in range(ll):
             G_icol += [mm + k * (nn + 1) for k in range(l)] * nn \
                   + list(range(mm + l * (nn + 1) + 1, mm + (l + 1) * (nn + 1)))
-            # irow += list(range(l * nn, (l + 1) * nn)) * (l + 1)
-            G_irow += [k for k in range(l * nn, (l + 1) * nn) for _ in range(l)]\
+            G_irow += \
+                [k for k in range(l * nn, (l + 1) * nn) for _ in range(l)] \
                 + list(range(l * nn, (l + 1) * nn))
             G_data += [-1.] * ((l + 1) * nn)
+            
         G_icol += list(range(mm + ll * (nn + 1) + 1))
         G_irow += list(range(ll * nn, ll * nn + mm + ll * (nn + 1) + 1))
         G_data += [-1.] * (mm + ll * (nn + 1) + 1)
@@ -356,16 +357,14 @@ class MADAnalyzer(_RiskAnalyzer):
         h_data = [0.] * (nn * ll + mm + ll * (nn + 1) + 1)
 
         # build A
-        A_icol = []
-        A_irow = []
-        A_data = []
+        A_icol = list(range(mm)) + [mm + ll * (nn + 1)]
+        A_irow = [0] * (mm + 1)
+        A_data = list(self.muk) + [-self.mu]
         for l in range(ll):
             A_icol += list(range(mm + l * (nn + 1), mm + (l + 1) * (nn + 1)))
-            A_irow += [l] * (nn + 1)
+            A_irow += [l + 1] * (nn + 1)
             A_data += [-1.] + [1. / nn] * nn
-        A_icol += list(range(mm)) + [mm + ll * (nn + 1)]
-        A_irow += [ll] * (mm + 1)
-        A_data += list(self.muk) + [-self.mu]
+        
         A_icol += list(range(mm)) + [mm + ll * (nn + 1)]
         A_irow += [ll + 1] * (mm + 1)
         A_data += [1.] * mm + [-1.]
@@ -374,27 +373,27 @@ class MADAnalyzer(_RiskAnalyzer):
         A = sps.coo_matrix((A_data, (A_irow, A_icol)), A_shape)
 
         # build b
-        b_data = [0.] * ll + [1., 0.]
+        b_data = [1.] + [0.] * (ll + 1)
 
         # calc
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
 
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"warning {res['status']}: {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']}")
             return np.array([np.nan] * mm)
 
         t = res['x'][-1]
-        # sharpe
+        # mMAD-Sharpe
         self.sharpe = 1. / res['pcost']
         # mMAD
         self.risk = res['pcost'] / t
-        # MAD
+        # delta-risk values
         self.primary_risk_comp = np.array([res['x'][mm + l * (nn + 1)] / t \
                                            for l in range(ll)])
-        # tMAD
-        self.secondary_risk_comp = np.cumsum(self.primary_risk_comp) \
-            - self.primary_risk_comp[0]
+        # delta-risk strikes
+        self.secondary_risk_comp = \
+            np.cumsum(np.insert(self.primary_risk_comp, 0, 0))[:-1]
         # optimal weights
         self.ww = np.array(res['x'][:mm] / t)
         self.ww.shape = mm
@@ -403,6 +402,7 @@ class MADAnalyzer(_RiskAnalyzer):
 
         return self.ww
 
+
     def _rr_max(self):
         # Order of variables:
         # w <- [0:mm]
@@ -424,10 +424,11 @@ class MADAnalyzer(_RiskAnalyzer):
         for l in range(ll):
             G_icol += [mm + k * (nn + 1) for k in range(l)] * nn \
                   + list(range(mm + l * (nn + 1) + 1, mm + (l + 1) * (nn + 1)))
-            # irow += list(range(l * nn, (l + 1) * nn)) * (l + 1)
-            G_irow += [k for k in range(l * nn, (l + 1) * nn) for _ in range(l)]\
+            G_irow += \
+                  [k for k in range(l * nn, (l + 1) * nn) for _ in range(l)] \
                   + list(range(l * nn, (l + 1) * nn))
             G_data += [-1.] * ((l + 1) * nn)
+            
         G_icol += list(range(mm + ll * (nn + 1)))
         G_irow += list(range(ll * nn, ll * nn + mm + ll * (nn + 1)))
         G_data += [-1.] * (mm + ll * (nn + 1))
@@ -439,40 +440,38 @@ class MADAnalyzer(_RiskAnalyzer):
         h_data = [0.] * (nn * ll + mm + ll * (nn + 1))
 
         # build A
-        A_icol = []
-        A_irow = []
-        A_data = []
+        A_icol = [mm + l * (nn + 1) for l in range(ll)]
+        A_irow = [0] * ll
+        A_data = list(self.coef)
+        
+        A_icol += list(range(mm))
+        A_irow += [1] * mm
+        A_data += [1.] * mm
         for l in range(ll):
             A_icol += list(range(mm + l * (nn + 1), mm + (l + 1) * (nn + 1)))
-            A_irow += [l] * (nn + 1)
+            A_irow += [l + 2] * (nn + 1)
             A_data += [-1.] + [1. / nn] * nn
-        A_icol += [mm + l * (nn + 1) for l in range(ll)]
-        A_irow += [ll] * ll
-        A_data += list(self.coef)
-        A_icol += list(range(mm))
-        A_irow += [ll + 1] * mm
-        A_data += [1.] * mm
 
         A_shape = (ll + 2, mm + (nn + 1) * ll)
         A = sps.coo_matrix((A_data, (A_irow, A_icol)), A_shape)
 
         # build b
-        b_data = [0.] * ll + [self.risk, 1.]
+        b_data = [self.risk, 1.]  + [0.] * ll
 
         # calc
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
 
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"warning {res['status']}: {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']}")
             return np.array([np.nan] * mm)
 
-        # MAD
+        # delta-risk values
         self.primary_risk_comp = np.array([res['x'][mm + l * (nn + 1)] \
                                            for l in range(ll)])
-        # tMAD
-        self.secondary_risk_comp = np.cumsum(self.primary_risk_comp) \
-            - self.primary_risk_comp[0]
+        # delta-risk strikes
+        self.secondary_risk_comp = \
+            np.cumsum(np.insert(self.primary_risk_comp, 0, 0))[:-1]
         # optimal weights
         self.ww = np.array(res['x'][:mm])
         self.ww.shape = mm
@@ -481,6 +480,7 @@ class MADAnalyzer(_RiskAnalyzer):
 
         return self.ww
 
+
     def _risk_averse(self):
         # Order of variables:
         # w <- [0:mm]
@@ -504,10 +504,10 @@ class MADAnalyzer(_RiskAnalyzer):
         for l in range(ll):
             G_icol += [mm + k * (nn + 1) for k in range(l)] * nn \
                   + list(range(mm + l * (nn + 1) + 1, mm + (l + 1) * (nn + 1)))
-            #irow += list(range(l * nn, (l + 1) * nn)) * (l + 1)
             G_irow += [k for k in range(l * nn, (l + 1) * nn) \
                        for _ in range(l)] + list(range(l * nn, (l + 1) * nn))
             G_data += [-1.] * ((l + 1) * nn)
+            
         G_icol += list(range(mm + ll * (nn + 1)))
         G_irow += list(range(ll * nn, ll * nn + mm + ll * (nn + 1)))
         G_data += [-1.] * (mm + ll * (nn + 1))
@@ -519,29 +519,26 @@ class MADAnalyzer(_RiskAnalyzer):
         h_data = [0.] * (nn * ll + mm + ll * (nn + 1))
 
         # build A
-        A_icol = []
-        A_irow = []
-        A_data = []
+        A_icol = list(range(mm))
+        A_irow = [0] * mm
+        A_data = [1.] * mm
         for l in range(ll):
             A_icol += list(range(mm + l * (nn + 1), mm + (l + 1) * (nn + 1)))
-            A_irow += [l] * (nn + 1)
+            A_irow += [l + 1] * (nn + 1)
             A_data += [-1.] + [1. / nn] * nn
-        A_icol += list(range(mm))
-        A_irow += [ll] * mm
-        A_data += [1.] * mm
-
+        
         A_shape = (ll + 1, mm + (nn + 1) * ll)
         A = sps.coo_matrix((A_data, (A_irow, A_icol)), A_shape)
 
         # build b
-        b_data = [0.] * ll + [1.]
+        b_data = [1.] + [0.] * ll
 
         # calc
         res = _lp_solver(self.method, c_data, G, h_data, A, b_data)
 
         self.status = res['status']
         if self.status != 0:
-            warnings.warn(f"warning {res['status']}: {res['infostring']}")
+            warnings.warn(f"Warning {res['status']}: {res['infostring']}")
             return np.array([np.nan] * mm)
 
         # optimal weights
@@ -551,11 +548,11 @@ class MADAnalyzer(_RiskAnalyzer):
         self.RR = np.dot(self.ww, self.muk)
         # mMAD
         self.risk = (res['pcost'] + self.RR) / self.Lambda
-        # MAD
+        # delta-risk values
         self.primary_risk_comp = np.array([res['x'][mm + l * (nn + 1)] \
                                            for l in range(ll)])
-        # tMAD
-        self.secondary_risk_comp = np.cumsum(self.primary_risk_comp) \
-            - self.primary_risk_comp[0]
+        # delta-risk strikes
+        self.secondary_risk_comp = \
+            np.cumsum(np.insert(self.primary_risk_comp, 0, 0))[:-1]
 
         return self.ww
