@@ -4,7 +4,7 @@ import matplotlib.pyplot as plt
 from collections import defaultdict
 import warnings
 
-from azapy.MkT.readMkTData import NYSEgen
+from azapy.MkT.MkTcalendar import NYSEgen
 
 class _RiskAnalyzer:
     """
@@ -35,36 +35,36 @@ class _RiskAnalyzer:
 
         Parameters
         ----------
-        mktdata : pandas.DataFrame, optional
+        `mktdata` : `pandas.DataFrame`, optional
             Historic daily market data for portfolio components in the format
-            returned by azapy.mktData function. The default is None.
-        colname : string, optional
+            returned by `azapy.mktData` function. The default is `None`.
+        `colname` : string, optional
             Name of the price column from mktdata used in the weights
             calibration. The default is 'adjusted'.
-        freq : string, optional
-            Rate of returns horizon in number of business day. it could be
-            'Q' for quarter or 'M' for month. The default is 'Q'.
-        hlength : float, optional
+        `freq` : string, optional
+            Rate of return horizon in number of business day. it could be
+            'Q' for quarter or 'M' for month. The default is `'Q'`.
+        `hlength` : float, optional
             History length in number of years used for calibration. A
             fractional number will be rounded to an integer number of months.
-            The default is 3.25 year.
-        calendar : np.busdaycalendar, optional
-            Business days calendar. If is it None then the calendar will be set
-            to NYSE business calendar.
-            The default is None.
-        rtype : string, optional
+            The default is `3.25` year.
+        `calendar` : `numpy.busdaycalendar`, optional
+            Business days calendar. If is it `None` then the calendar will be 
+            set to NYSE business calendar. The default is `None`.
+        `rtype` : string, optional
             Optimization type. Possible values \n
-                "Risk" : minimization of dispersion (risk) measure.\n
-                "Sharpe" : maximization of generalized Sharpe ratio.\n
-                "Sharpe2" : alternative computation of generalized Sharpe
+                'Risk' : minimization of dispersion (risk) measure for 
+                targetd expected rate of return.\n
+                'Sharpe' : maximization of generalized Sharpe ratio.\n
+                'Sharpe2' : minimization of inverse of generalized Sharpe
                 ratio.\n
-                "MinRisk" : optimal portfolio with minimum dispersion (risk)
-                value.\n
-                "InvNRisk" : optimal portfolio with the same dispersion (risk)
-                value as equally weighted portfolio. \n
-                "RiskAverse" : optimal portfolio for a fixed risk aversion
-                coefficient.
-            The default is "Sharpe".
+                'MinRisk' : minimum dispersion (risk) portfolio.\n
+                'InvNrisk' : optimal portfolio with the same dispersion (risk)
+                value as a benchmark portfolio 
+                (e.g. equal weighted portfolio).\n
+                'RiskAverse' : optimal portfolio for a fixed risk-aversion
+                factor.
+            The default is 'Sharpe'.
 
         Returns
         -------
@@ -79,8 +79,11 @@ class _RiskAnalyzer:
         self.mm = None
         self.muk = None
         self.mu = None
+        
         self.alpha = [1.]
         self.coef = [1.]
+        
+        self.Lambda = None
 
         self.risk = None
         self.primary_risk_comp = None
@@ -97,47 +100,81 @@ class _RiskAnalyzer:
 
         self.rng = None
         self.set_random_seed()
+        
+        self.method = None
 
-    def getWeights(self, mu, rrate=None, rtype=None, d=1):
+        
+    def getWeights(self, rtype=None, mu=None, d=1, mu0=0., aversion=None,
+                   ww0=None, rrate=None ):  
         """
         Computes the optimal portfolio weights.
 
         Parameters
         ----------
-        mu : float
-            Rate of reference. Its meaning depends on the optimization
-            criterion. For rtype set to\n
-                "Risk" : mu is the targeted portfolio rate of returns.\n
-                "Sharpe" and "Sharpe2" : mu is the risk-free rate.\n
-                "MinRisk" and "InvNRisk": mu is ignored. \n
-                "RiskAverse" : mu is the Lambda aversion coefficient.
-        rrate : pandas.DataFrame, optional
-            The portfolio components historical rates of returns.
-            If it is not None, it will overwrite the rrate computed in the
-            constructor from mktdata. The default is None.
-        rtype : string, optional
-            Optimization type. If is not None it will overwrite the value
-            set by the constructor. The default is None.
-        d : int, optional
-            Frontier type. Has effect only if rtype="Risk". A value of 1 will
+        `rtype` : str, optional
+            Optimization type. If is not `None` it will overwrite the value
+            set by the constructor. The default is `None`.
+        `mu` : float, optional
+            Targeted portfolio expected rate of return. 
+            Relevant only if `rtype='Risk'`
+            The default is `None`.
+        `d` : int, optional
+            Frontier type. Active only if `rtype='Risk'`. A value of `1` will
             trigger the evaluation of optimal portfolio along the efficient
-            frontier. Otherwise it will find the portfolio with the lowest
+            frontier. Otherwise, it will find the portfolio with the lowest
             rate of return along the inefficient portfolio frontier.
-            The default is 1.
+            The default is `1`.
+        `mu0` : float, optional
+            Risk-free rate accessible to the investor.
+            Relevant only if `rype='Sharpe'` or `rtype='Sharpe2'`.
+            The default is `0`.
+        `aversion` : float, optional
+            The value of the risk-aversion coefficient.
+            Must be positive. Relevant only if `rtype='RiskAverse'`.
+            The default is `None`.
+        `ww0` : list (also `numpy.array` or `pandas.Series`), optional
+            Targeted portfolio weights. 
+            Relevant only if `rype='InvNrisk'`.
+            Its length must be equal to the number of
+            symbols in `rrate` (mktdata). 
+            All weights must be >= 0 with sum > 0.
+            If it is a list or a `numpy.array` then the weights are assumed to
+            by in order of `rrate.columns`. If it is a `pandas.Series` then 
+            the index should be compatible with the `rrate.columns` or mktdata 
+            symbols (same symbols, not necessary in the same order).
+            If it is `None` then it will be set to equal weights.
+            The default is `None`.
+        `rrate` : `pandas.DataFrame`, optional
+            The portfolio components historical rates of returns.
+            If it is not `None`, it will overwrite the rrate computed in the
+            constructor from mktdata. The default is `None`. 
 
-        Returns
+        Raises
+        ------
+        ValueError
+            Wrong rtype value.
+
+       Returns
         -------
-        pandas.Series
-            Portfolio weights.
+        `pandas.Series` with portfolio weights.
         """
+        self.ww = None
+        self.risk = None
+        self.primary_risk_comp = None
+        self.secondary_risk_comp = None
+        self.sharpe = None
+        self.RR = None
+        
         if rrate is not None:
             self.set_rrate(rrate)
 
-        if rtype is not None:
+        if not rtype is None:
             self.set_rtype(rtype)
-
+            
         if self.rtype == "Risk":
-            if mu > self.muk.max():
+            if mu is None:
+                raise ValueError("for rtype='Risk' mu must have a value")
+            elif mu > self.muk.max():
                 self.mu = self.muk.max()
             elif mu < self.muk.min():
                 self.mu = self.muk.min()
@@ -145,57 +182,91 @@ class _RiskAnalyzer:
                 self.mu = mu
 
             self._risk_min(d=d)
-        elif self.rtype == "Sharpe":
-            if mu > self.muk.max():
+            
+        elif self.rtype == 'Sharpe':           
+            if mu0 is None:
+                raise ValueError("for rtype='Sharpe' mu0 must have a value")
+            if mu0 > self.muk.max():
                 self.mu = self.muk.max() * 0.999
             else:
-                self.mu = mu
+                self.mu = mu0
 
             self._sharpe_max()
-        elif self.rtype == "MinRisk":
-            self.mu = self.muk.min()
-
-            self._risk_min(d=1)
-        elif self.rtype == "Sharpe2":
-            if mu > self.muk.max():
+            
+        elif self.rtype == 'Sharpe2':          
+            if mu0 is None:
+                raise ValueError("for rtype='Sharpe2' mu0 must have a value")
+            if mu0 > self.muk.max():
                 self.mu = self.muk.max() * 0.999
             else:
-                self.mu = mu
+                self.mu = mu0
 
-            self._sharpe_inv_min()
-        elif self.rtype == "InvNrisk":
-            ww = np.array([1.] * len(self.rrate.columns))
-            ww = ww / np.sum(ww)
+            self._sharpe_inv_min() 
+            
+        elif self.rtype == "MinRisk":
+            self.mu = self.muk.min()
+            self._risk_min(d=1)
+            
+        elif self.rtype == "InvNrisk":    
+            if ww0 is None:
+                ww = np.full(len(self.rrate.columns), 
+                             1/len(self.rrate.columns))
+            else:
+                if isinstance(ww0, pd.core.series.Series):
+                    ww = np.array(ww0[self.rrate.columns])
+                else:
+                    ww = np.array(ww0)
+                if any(ww < 0.):
+                    raise ValueError("All ww0 must be non-negative")
+                sww = ww.sum()
+                if sww <= 0:
+                    raise ValueError("At least one ww0 must be non zero")
+                ww = ww / sww
+                
             self.getRisk(ww)
-            self._rr_max()
-        elif self.rtype == "RiskAverse":
-            self.Lambda = mu
+            self._rr_max()  
+            
+        elif self.rtype == "RiskAverse":          
+            if aversion is None:
+                raise ValueError("for rtype='RiskAverse'"
+                                 + " aversion must have a value")
+            elif aversion < 0:
+                raise ValueError("aversion must be positive")
+                
+            self.Lambda = aversion
             self._risk_averse()
+            
         else:
-            print(f"should not be here!! Unknown rtype {rtype}")
-            return np.nan
-
-        self.ww = pd.Series(self.ww, index=self.rrate.columns)
+            raise ValueError("you should not be here")
+        
+        if self.status != 0:
+            warnings.warn(f"Warning: status {self.status} for {self.rtype}"
+                          + " is not 0")
+            
+        self.ww = pd.Series(self.ww, index=self.rrate.columns, dtype='float64')
         return self.ww
 
+
     def getRisk(self, ww, rrate=None):
         """
-        Returns the value of the dispersion (risk) measure for a give portfolio.
+        Returns the value of the dispersion (risk) measure for a give 
+        portfolio.
 
         Parameters
         ----------
-        ww : list (np.array or pandas.Series)
+        `ww` : list (`np.array` or `pandas.Series`)
             Portfolio weights. Its length must be equal to the number of
-            symbols in rrate (mktdata). All weights must by >0.
-            If it is a list or a np.array then the weights are assumed to
-            by in order of rrate.columns. If it is a pd.Series the index
-            should be compatible with the rrate.columns or mktdata symbols
-            (not necessary in the same order).
-        rrate : pd.DataFrame, optional
+            symbols in `rrate` (mktdata). All weights must be >=0 with 
+            sum > 0.
+            If it is a list or a `numpy.array` then the weights are assumed to
+            by in order of `rrate.columns`. If it is a `pandas.Series` than 
+            the index should be compatible with the `rrate.columns` or mktdata 
+            symbols (not necessary in the same order).
+        `rrate` : `pandas.DataFrame`, optional
             Contains the portfolio components historical
-            rates of returns. If it is not None, it will overwrite the
+            rates of returns. If it is not `None`, it will overwrite the
             rrate computed in the constructor from mktdata.
-            The default is None.
+            The default is `None`.
 
         Returns
         -------
@@ -210,7 +281,10 @@ class _RiskAnalyzer:
         w = np.array(ww)
         if any(w < 0.):
             raise ValueError("All ww must be non-negative")
-        w = w / w.sum()
+        sw = w.sum()
+        if sw <= 0:
+            raise ValueError("At least one ww must be positive")
+        w = w / sw
 
         prate = np.dot(self.rrate, w)
 
@@ -233,66 +307,90 @@ class _RiskAnalyzer:
 
         return self.risk
 
-    def getPositions(self, mu, rtype=None, nshares=None, cash=0, ww=None):
+        
+    def getPositions(self, nshares=None, cash=0, ww=None, rtype=None, 
+                     mu=None, mu0=0., aversion=None, ww0=None, ):
         """
-        Computes the number of shares according to the weights
+        Computes the rebalanced number of shares.
 
         Parameters
         ----------
-        mu : float
-            Rate of reference. Its meaning depends on the optimization
-            criterion. For rtype set to\n
-                "Risk" : mu is the targeted portfolio rate of returns.\n
-                "Sharpe" and "Sharpe2" : mu is the risk-free rate.\n
-                "MinRisk" and "InvNRisk": mu is ignored. \n
-                "RiskAverse" : mu is the Lambda aversion coefficient.
-        rtype : string, optional
-            Optimization type. If is not None it will overwrite the value
-            set by the constructor. The default is None.
-        nshares : pd.Series, optional
+        `nshares` : `panda.Series`, optional
             Initial number of shares per portfolio component.
             A missing component
-            entry will be considered 0. A None value assumes that all
+            entry will be considered 0. A `None` value assumes that all
             components entries are 0. The name of the components must be
-            present in the mrkdata. The default is None.
-        cash : float, optional
+            present in the mrkdata. The default is `None`.
+        `cash` : float, optional
             Additional cash to be added to the capital. A
             negative entry assumes a reduction in the total capital
-            available for rebalance. The default is 0.
-        ww : pd.Series, optional
-            External portfolio weights. If it not set to None these
-            weights will overwrite the calibrated weights.
-            The default is None.
+            available for rebalance. The total capital cannot be < 0.
+            The default is 0. 
+        `ww` : `panda.Series`, optional
+            External overwrite portfolio weights. 
+            If it not set to `None` these
+            weights will overwrite the calibrated.
+            The default is `None`. 
+        `rtype` : str, optional
+            Optimization type. If is not `None` it will overwrite the value
+            set by the constructor. The default is `None`.
+        `mu` : float, optional
+            Targeted portfolio expected rate of return. 
+            Relevant only if `rtype='Risk'`
+            The default is `None`.
+        `mu0` : float, optional
+            Risk-free rate accessible to the investor.
+            Relevant only if `rype='Sharpe'` or `rtype='Sharpe2'`.
+            The default is `0`.
+        `aversion` : float, optional
+            The value of the risk-aversion coefficient.
+            Must be positive. Relevant only if `rtype='RiskAvers'`.
+            The default is `None`.
+        `ww0` : list (also `np.array` or `pandas.Series`), optional
+            Targeted portfolio weights 
+            Relevant only if `rype='InvNrisk'`.
+            Its length must be equal to the number of
+            symbols in `rrate` (mktdata). 
+            All weights must be >= 0 with sum > 0.
+            If it is a list or a `numpy.array` then the weights are assumed to
+            by in order of `rrate.columns`. If it is a `pandas.Series` then the 
+            index should be compatible with the `rrate.columns` or mktdata 
+            symbols (same symbols, not necessary in the same order).
+            If it is `None` then it will be set to equal weights.
+            The default is `None`.
 
         Returns
         -------
-        pd.DataFrame: the rolling information.
+        `pandas.DataFrame`: the rolling information.
 
         Columns:
 
             - "old_nsh" :
-                initial number of shares per portfolio component as well as
-                additional cash position. These are present in the input.
+                initial number of shares per portfolio component and
+                the additional cash. These are input values.
             - "new_nsh" :
                 the new number of shares per component plus the residual
                 cash (due to the rounding to an integer number of shares).
                 A negative entry means that the investor needs to add more
-                cash in order to cover for the number of share roundups.
+                cash to cover for the roundup shortfall.
                 It has a small value.
             - "diff_nsh" :
                 the number of shares that needs to be both/sold in order
                 to rebalance the portfolio positions.
             - "weights" :
-                portfolio weights used for rebalance. The cash entry is
+                portfolio weights used for rebalancing. The cash entry is
                 the new portfolio value (invested capital).
             - "prices" :
-                the share prices used for rebalance evaluations.
+                the share prices used for rebalances evaluations.
 
         Note: Since the prices are closing prices, the rebalance can be
-        executed next business. Additional cash slippage may occur due
+        computed after the market close and before the 
+        trading execution (next day). 
+        Additional cash slippage may occur due
         to share price differential between the previous day closing and
         execution time.
         """
+       
         if rtype is not None:
             self.set_rtype(rtype)
 
@@ -308,10 +406,11 @@ class _RiskAnalyzer:
 
         cap = ns.dot(pp) + cash
         if ww is None:
-            ww = self.getWeights(mu, rtype=rtype)
+            ww = self.getWeights(rtype=rtype, mu=mu, mu0=mu0, ww0=ww0, 
+                                 aversion=aversion)
         else:
-            ww0 = pd.Series(0, index=self.rrate.columns)
-            ww = ww0.add(ww, fill_value=0)
+            wwp = pd.Series(0, index=self.rrate.columns)
+            ww = wwp.add(ww, fill_value=0)
             if len(self.rrate.columns) == len(ns):
                 raise ValueError(f"ww must be among {self.rrate.columns}")
 
@@ -326,7 +425,7 @@ class _RiskAnalyzer:
         res = pd.DataFrame({'old_nsh': ns,
                             'new_nsh': newns,
                             'diff_nsh': newns - ns,
-                            'weights' : ww.round(6),
+                            'weights': ww.round(6),
                             'prices': pp})
 
         return res
@@ -339,9 +438,9 @@ class _RiskAnalyzer:
 
         Parameters
         ----------
-        rrate : pd.DataFrame
-            Portfolio components historical rates of returns, where the
-            columns are "date", "symbol1", "symbol2", etc.
+        `rrate` : `pandas.DataFrame`
+            Portfolio components historical rates of returns. The
+            columns are: "date", "symbol1", "symbol2", etc.
         Returns
         -------
         None
@@ -350,6 +449,7 @@ class _RiskAnalyzer:
         self.muk = rrate.mean()
         self.rrate = rrate - self.muk
 
+
     def set_mktdata(self, mktdata, colname='adjusted',
                     freq='Q', hlength=3.25, calendar=None):
         """
@@ -358,23 +458,23 @@ class _RiskAnalyzer:
 
         Parameters
         ----------
-        mktdata : pandas.DataFrame
+        `mktdata` : `pandas.DataFrame`
             Historic daily market data for portfolio components in the format
-            returned by azapy.mktData function.
-        colname : string, optional
+            returned by `azapy.mktData` function.
+        `colname` : str, optional
             Name of the price column from mktdata used in the weights
             calibration. The default is 'adjusted'.
-        freq : string, optional
-            Rate of returns horizon in number of business day. it could be
+        `freq` : str, optional
+            Rate of returns horizon in number of business day. It could be
             'Q' for quarter or 'M' for month. The default is 'Q'.
-        hlength : float, optional
+        `hlength` : float, optional
             History length in number of years used for calibration. A
             fractional number will be rounded to an integer number of months.
-            The default is 3.25
-        calendar : np.busdaycalendar, optional
-            Business days calendar. If is it None then the calendar will be set
-            to NYSE business calendar.
-            The default is None.
+            The default is `3.25`.
+        `calendar` : `numpy.busdaycalendar`, optional
+            Business days calendar. If is it `None` then the calendar will be 
+            set to NYSE business calendar.
+            The default is `None`.
 
         Returns
         -------
@@ -400,6 +500,7 @@ class _RiskAnalyzer:
 
         self.set_rrate(rrate)
 
+
     def set_rtype(self, rtype):
         """
         Sets the optimization type. It will overwrite the value set in the
@@ -407,7 +508,7 @@ class _RiskAnalyzer:
 
         Parameters
         ----------
-        rtype : string
+        `rtype` : str
             Optimization type.
         Returns
         -------
@@ -429,58 +530,63 @@ class _RiskAnalyzer:
 
         Parameters
         ----------
-        efficient : int, optional
+        `efficient` : int, optional
             Number of points along the optimal frontier (equally spaced along
-            the rate of returns). The default is 20.
-        inefficient : int, optional
+            the rate of return axis). The default is `20`.
+        `inefficient` : int, optional
             Number of points along the inefficient frontier (equally spaced
-            along the rate of returns). The default is 20.
-        musharpe : float, optional
+            along the rate of returns axis). The default is `20`.
+        `musharpe` : float, optional
             Risk-free rate value used in the evaluation of
-            generalized Sharpe ratio. The default is 0.
-        component : boolean, optional
-            If True the portfolios containing a single component are evaluated
-            and added to the plot for references. The default is True.
-        randomport : int, optional
+            generalized Sharpe ratio. The default is `0`.
+        `component` : Boolean, optional
+            If `True` the portfolios containing a single component are 
+            evaluated and added to the plot for references. 
+            The default is `True`.
+        `randomport` : int, optional
             Number of portfolios with random weights (inefficient) to be
-            evaluate and added to the plot for reference. The default is 20.
-        inverseN : boolean, optional
-            If True the equally weighted portfolio and the optimal portfolio
+            evaluate and added to the plot for reference. The default is `20`.
+        `inverseN` : boolean, optional
+            If `True` then the equally weighted portfolio and the optimal 
+            portfolio
             with the same dispersion (risk) value are evaluated and added to
-            the plot. The default is True.
-        fig_type : string, optional
+            the plot. The default is `True`.
+        `fig_type` : str, optional
             Graphical representation format.
             If it is set to 'RR_risk' the data is plotted in the rate of return
             vs dispersion representation, otherwise the Sharpe vs rate of
-            return will be used. The default is 'RR_risk'.
-        options : dictionary, optional
+            return will be used. The default is `'RR_risk'`.
+        `options` : dict, optional
             Additional graphical parameters. Relevant keys are:\n
                 'title' : The default is 'Portfolio frontiers'.\n
-                'xlabel' : The default is 'risk' if fig_type='RR_risk'
-                and 'rate of returns' otherwise.\n
-                'ylabel' : The default is 'rate of returns' if
-                fig_type='RR_risk' and 'sharpe' otherwise.\n
-                'tangent' : Boolean flag. If set to True the tangent
+                'xlabel' : The default is `'risk'` if `fig_type='RR_risk'`
+                and `'rate of returns'` otherwise.\n
+                'ylabel' : The default is `'rate of returns'` if
+                `fig_type='RR_risk'` and `'sharpe'` otherwise.\n
+                'tangent' : Boolean flag. If set to `True` the tangent
                 (to sharpe point) is added. It has effect only  if
-                fig_type='RR_risk'. The default is True.
-        saveto : string, optional
+                `fig_type='RR_risk'`. The default is `True`.
+        `saveto` : str, optional
             File name to save the figure. The extension dictates the format:
             png, pdf, svg, etc. For more details see the mathplotlib
-            documentation for savefig. The default is None.
-        data : dictionary, optional
-            Numerical data to construct the plot. If it is not None it
+            documentation for savefig. The default is `None`.
+        `data` : dict, optional
+            Numerical data to construct the plot. If it is not `None` it
             will take precedence and no other numerical evaluations will be
             performed. It is meant to produce different plot representations
-            without reevaluations. The default is None.
+            without reevaluations. The default is `None`.
 
         Returns
         -------
-        dictionary
+        `dict`
             Numerical data used to make the plots. It can be passed back to
             reconstruct the plots without reevaluations.
         """
         if data is not None:
             data['saveto'] = saveto
+            
+            data['options'] = options
+                
             if fig_type == 'RR_risk':
                 self._plot_f1(data)
             else:
@@ -493,7 +599,7 @@ class _RiskAnalyzer:
         # min risk
         res['risk_min'] = defaultdict(lambda x=0: None)
         rr = self.muk.min()
-        ww_min = self.getWeights(mu=rr, rtype='Risk')
+        ww_min = self.getWeights(rtype='Risk', mu=rr)
         risk_min = self.risk
         rr_min = self.RR
         res['risk_min']['risk'] = risk_min
@@ -509,7 +615,7 @@ class _RiskAnalyzer:
             ww_eff = []
             RR_eff = []
             for rr in rr_eff:
-                ww_eff.append(self.getWeights(mu=rr, rtype='Risk'))
+                ww_eff.append(self.getWeights(rtype='Risk', mu=rr))
                 risk_eff.append(self.risk)
                 RR_eff.append(self.RR)
             res['efficient']['risk'] = np.array(risk_eff)
@@ -525,7 +631,7 @@ class _RiskAnalyzer:
             ww_ineff = []
             RR_ineff = []
             for rr in rr_ineff:
-                ww_ineff.append(self.getWeights(mu=rr, rtype='Risk', d=-1))
+                ww_ineff.append(self.getWeights(rtype='Risk', mu=rr, d=-1))
                 risk_ineff.append(self.risk)
                 RR_ineff.append(self.RR)
             res['inefficient']['risk'] = np.array(risk_ineff)
@@ -537,7 +643,7 @@ class _RiskAnalyzer:
         res['sharpe']['mu'] = musharpe
         if not np.isnan(musharpe):
             if musharpe > self.muk.max(): musharpe = self.muk.max() * 0.9999
-            ww_sharpe = self.getWeights(mu=musharpe, rtype='Sharpe')
+            ww_sharpe = self.getWeights(rtype='Sharpe', mu0=musharpe)
             rr_sharpe = self.RR
             risk_sharpe = self.risk
             sharpe = self.sharpe
@@ -597,13 +703,13 @@ class _RiskAnalyzer:
             rr_n.append(self.RR)
             ww_n.append(wwn)
             # optimal 1/N risk portfolio
-            ww_n.append(self.getWeights(0., rtype="InvNrisk"))
+            ww_n.append(self.getWeights(rtype="InvNrisk"))
             risk_n.append(self.risk)
             rr_n.append(self.RR)
             res['inverseN']['risk'] = np.array(risk_n)
             res['inverseN']['rr'] = rr_n
             res['inverseN']['ww'] = ww_n
-            res['inverseN']['label'] = ['1/N', 'InvNRisk']
+            res['inverseN']['label'] = ['1/N', 'InvNrisk']
 
         # Plot
         if fig_type == 'RR_risk':
@@ -612,6 +718,7 @@ class _RiskAnalyzer:
             self._plot_f2(res)
         return res
 
+
     def _plot_f1(self, res):
         fig = plt.figure()
         ax = fig.add_subplot(111)
@@ -692,6 +799,7 @@ class _RiskAnalyzer:
             plt.savefig(res['saveto'])
         plt.show()
 
+
     def _plot_f2(self, res):
         fig = plt.figure()
         ax = fig.add_subplot(111)
@@ -765,15 +873,17 @@ class _RiskAnalyzer:
             plt.savefig(res['saveto'])
         plt.show()
 
+
     def set_random_seed(self, seed = 42):
         """
-        Sets the seed for Dirichlet random generator used in viewFrontiers.
+        Sets the seed for Dirichlet random generator used in viewFrontiers
+        to select the inefficient portfolios.
 
         Parameters
         ----------
-        seed : int, optional
+        `seed` : int, optional
             The random generator seed, in case you want to set it to a weird
-            value other than 42 :). The default is 42.
+            value other than 42 :). The default is `42`.
 
         Returns
         -------
@@ -781,8 +891,35 @@ class _RiskAnalyzer:
         """
         self.rng = np.random.RandomState(seed)
 
+
     def _ww_gen(self):
         return self.rng.dirichlet([0.5] * self.mm)
+    
+    
+    def _set_lp_method(self, method):
+        lp_methods = ['ecos', 'highs-ds', 'highs-ipm', 'highs',
+                       'interior-point', 'glpk', 'cvxopt']
+        if not method in lp_methods:
+            raise ValueError("unknown method {method} - "
+                             + f"must be one of {lp_methods}")
+        self.method = method
+        
+        
+    def _set_qp_method(self, method):
+        qp_methods = ['ecos', 'cvxopt']
+        if not method in qp_methods:
+            raise ValueError("unknown method {method} - "
+                             + f"must be one of {qp_methods}")
+        self.method = method
+        
+        
+    def _set_socp_method(self, method):
+        socp_methods = ['ecos', 'cvxopt']
+        if not method in socp_methods:
+            raise ValueError("unknown method {method} - "
+                             + f"must be one of {socp_methods}")
+        self.method = method
+        
 
     # to be implemented in the deriv class
     def _risk_calc(self, prate, alpha):
